/* 
 *  ===================================================================
 *  DemoWheel.java : Simple GUI with canvas to draw a wheel assembled
 *  from triangles. The canvas is connected to a mouse listener
 *  and buttons connected to button listeners.
 * 
 *  Written By: Mark Austin                               December 2007
 *  ===================================================================
 */ 

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class DemoWheel extends JApplet {

    // Create reference to graphics contianer and graphics screen ...

    Container container = null;
    DemoGraphicsScreen gs;

    public void init() {

      // 1. Get a handle on the applet's content pane.
   
      container = this.getContentPane();

      // 2. Create a graphics screen and mouse listener,
      //    and add content pane to center of applet.

      DemoGraphicsScreen gs = new DemoGraphicsScreen();
      gs.addMouseListener( new GraphicsListener( gs ));

      // 3. Create buttons and button listeners.

      JButton buttons[] = new JButton [3]; 

      buttons[0] = new JButton ("Clear");
      buttons[0].addActionListener( new ButtonAction( buttons[0], gs ));
      buttons[1] = new JButton ("Draw Wheel");
      buttons[1].addActionListener( new ButtonAction( buttons[1], gs ));
      buttons[2] = new JButton ("Area");
      buttons[2].addActionListener( new ButtonAction( buttons[2], gs ));

      // 4. Create panel. Add buttons to panel.

      Panel p1 = new Panel();
      for(int ii = 1; ii <= 3; ii++ )
          p1.add( buttons[ii-1] );

      // 5. Position canvas and panel within the frame.

      container.add( "South", p1 );
      container.add("Center", gs );
   }
}

/*
 *  =================================================================
 *  This class listens for action events associated with the buttons.
 *  =================================================================
 */

class ButtonAction implements ActionListener {
      private JButton b;
      private DemoGraphicsScreen gs;

      public ButtonAction ( JButton b, DemoGraphicsScreen gs ) {
         this.b  = b;
         this.gs = gs;
      }

      public void actionPerformed ( ActionEvent e ) {

         String s = new String(e.getActionCommand());
         System.out.println(s);

         if( s.compareTo("Clear") == 0 ) {
             gs.clear();
         }
         if( s.compareTo("Draw Wheel") == 0 ) {
             gs.gsDrawWheel();
         }
         if( s.compareTo("Area") == 0 ) {
             gs.gsDrawArea();
         }
      }
}

/*
 *  ==============================================================
 *  This class listens for mouse events associated with the canvas
 *  ==============================================================
 */

class GraphicsListener implements MouseListener {
      private DemoGraphicsScreen gs;

      public GraphicsListener( DemoGraphicsScreen gs ) {
         this.gs = gs;
      }

      public void mouseReleased ( MouseEvent e ) {

          // Get and save new coordinates...

          gs.iXcoord = e.getX();
          gs.iYcoord = e.getY();

          // Print cursor coordinates and distance to triangle...

          gs.drawText();
      }

      public void mouseClicked ( MouseEvent e ) {};
      public void mouseEntered ( MouseEvent e ) {};
      public void mouseExited  ( MouseEvent e ) {};
      public void mousePressed ( MouseEvent e ) {};
}

/*
 *  =======================
 *  Create graphics screen.
 *  =======================
 */

class DemoGraphicsScreen extends JPanel {
    int iXcoord, iYcoord;
    String cursorMessage   = null;
    String areaMessage     = null;

    private Dimension size;
    private Graphics    gs;

    // Create references to arraylist of triangles ....

    String sName;
    List parts = new ArrayList();

    // Constructor method

    public DemoGraphicsScreen() {
      double dX1, dX2, dX3;
      double dY1, dY2, dY3;
      double dMinWheelRadius =  50;
      double dMaxWheelRadius = 150;
      int    iNoAngularIntervals = 16;
      int     iNoRadialIntervals =  4;

      double dCenterX = 200;
      double dCenterY = 200;

      for ( int i = 1; i <= iNoAngularIntervals; i = i + 1 ) {
      for ( int j = 1; j <=  iNoRadialIntervals; j = j + 1 ) {

            double dTheta1 = ((i-1)/((double) iNoAngularIntervals))*2.0*Math.PI;
            double dTheta2 =     (i/((double) iNoAngularIntervals))*2.0*Math.PI;

            double dMinRadius = dMinWheelRadius +
                   (j-1)/((double) iNoRadialIntervals)*(dMaxWheelRadius-dMinWheelRadius);
            double dMaxRadius = dMinWheelRadius +
                     (j)/((double) iNoRadialIntervals)*(dMaxWheelRadius-dMinWheelRadius);

            // Create first triangle ....

            dX1 = dCenterX + dMinRadius*Math.sin(dTheta1);
            dY1 = dCenterY + dMinRadius*Math.cos(dTheta1);
            dX2 = dCenterX + dMaxRadius*Math.sin(dTheta1);
            dY2 = dCenterY + dMaxRadius*Math.cos(dTheta1);
            dX3 = dCenterX + dMaxRadius*Math.sin(dTheta2);
            dY3 = dCenterY + dMaxRadius*Math.cos(dTheta2);
            Triangle a = new Triangle( dX1, dY1, dX2, dY2, dX3, dY3 );

            // Create second triangle ....

            dX1 = dCenterX + dMinRadius*Math.sin(dTheta1);
            dY1 = dCenterY + dMinRadius*Math.cos(dTheta1);
            dX2 = dCenterX + dMinRadius*Math.sin(dTheta2);
            dY2 = dCenterY + dMinRadius*Math.cos(dTheta2);
            dX3 = dCenterX + dMaxRadius*Math.sin(dTheta2);
            dY3 = dCenterY + dMaxRadius*Math.cos(dTheta2);
            Triangle b = new Triangle( dX1, dY1, dX2, dY2, dX3, dY3 );

            // Add triangles to array list ....

            parts.add( a );
            parts.add( b );
      }
      }
    }

    //  Use the paint() method to display the (x,y) coordinates of the
    //  mouse when it is clicked...

    public void paint(Graphics g) {
       size = getSize();
       drawText();
    }

    // Draw cursor coordinates on the canvas ....

    public void drawText() {
       gs = getGraphics();
       Graphics2D g2D = (Graphics2D) gs;

       Node pt1 = new Node( iXcoord, iYcoord );

       // Display of cursor coordinates ....

       if ( cursorMessage != null ) {
          g2D.setColor( Color.white );
          g2D.drawString( cursorMessage, 15, getHeight()-50);
       }

       cursorMessage = "Cursor: " + iXcoord + ", " + iYcoord;
       g2D.setColor(Color.blue);
       g2D.drawString( cursorMessage, 15, getHeight()-50);
    }
    
    // Get graphics and fill in background ....

    public void clear () {
        gs = getGraphics();
        Graphics2D g2D = (Graphics2D) gs;

        g2D.setColor( Color.white );
        g2D.fillRect( 0, 0, size.width-1, size.height-1 );
    }

    // Method to draw triangle ....

    public void gsDrawWheel () {
        gs = getGraphics();
        Graphics2D g2D = (Graphics2D) gs;

        // Walk along array list and draw triangle objects ....

        Iterator iterator1 = parts.iterator();
        while ( iterator1.hasNext() != false ) {
            Triangle tp = (Triangle) iterator1.next();

            // Draw triangle nodes and edges ....

            g2D.setColor( Color.blue );
            g2D.drawLine ( (int) tp.node1.dX, (int) tp.node1.dY,
                           (int) tp.node2.dX, (int) tp.node2.dY );
            g2D.drawLine ( (int) tp.node2.dX, (int) tp.node2.dY,
                           (int) tp.node3.dX, (int) tp.node3.dY );
            g2D.drawLine ( (int) tp.node1.dX, (int) tp.node1.dY,
                           (int) tp.node3.dX, (int) tp.node3.dY );

            g2D.fillRect( (int) (tp.node1.dX-3), (int) (tp.node1.dY-3), 6, 6);
            g2D.fillRect( (int) (tp.node2.dX-3), (int) (tp.node2.dY-3), 6, 6);
            g2D.fillRect( (int) (tp.node3.dX-3), (int) (tp.node3.dY-3), 6, 6);
        }
    }

    // Method to compute and draw cross-section area of wheel...

    public void gsDrawArea () {
        gs = getGraphics();
        Graphics2D g2D = (Graphics2D) gs;
        double dArea = 0.0;

        // Walk along array list and compute total area...

        Iterator iterator1 = parts.iterator();
        while ( iterator1.hasNext() != false ) {
            Triangle tp = (Triangle) iterator1.next();
            dArea = dArea + tp.area();
        }

        // Display of area on canvas ...

        if ( areaMessage != null ) {
           g2D.setColor( Color.white );
           g2D.drawString( areaMessage, 15, getHeight()-80);
        }

        areaMessage = "Cross Section Area = " + dArea;
        g2D.setColor(Color.blue);
        g2D.drawString( areaMessage, 15, getHeight()-80);
    }
}
